/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package net.buildism;

import net.buildism.ui.UI;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.Transparency;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.BufferedImage;
import java.awt.image.VolatileImage;
import javax.swing.JDesktopPane;
import net.buildism.input.XKeyListener;
import net.buildism.input.XMouseListener;
import net.buildism.input.XMouseMotionListener;
import net.buildism.input.XMouseWheelListener;


public class GamePanel extends JDesktopPane{
    private boolean chatVisible = false;
    private boolean initialResize = false;
    private int width = 1024;
    private int height = 768;
    private MainView view;


    private Cursor cursor;

    private TexturePaint loadingPaint;

    private VolatileImage buffer;
    private Graphics bufGraphics;

    protected void setStartSize(int w, int h)
    {
        this.width = w;
        this.height = h;
    }
    public void setup()
    {
        setPreferredSize(new Dimension(640, 480));
        setIgnoreRepaint(true);
        addMouseListener(new XMouseListener());
        addMouseMotionListener(new XMouseMotionListener());
        addKeyListener(new XKeyListener());
        addMouseWheelListener(new XMouseWheelListener());
        addComponentListener(new ComponentListener()
        {

            public void componentResized(ComponentEvent e) {
                if(Main.canUseTools || !initialResize)
                {
                    width = getWidth();
                    height = getHeight();
                    view.setSize(width, height);
                    if(width <= 0 || height <= 0)
                        return;
                    rebuildImage();
                    
                    if(Main.getGame() != null && Main.getGame().getCurrentWorld() != null)
                        view.updatePaint(Main.getGame().getCurrentWorld());
                    view.updateBackground();
                    repaint();
                }
                initialResize = true;
            }

            public void componentMoved(ComponentEvent e) {

            }

            public void componentShown(ComponentEvent e) {

            }

            public void componentHidden(ComponentEvent e) {

            }

        });
    }

    public void setView(MainView v)
    {
        view = v;
    }
    public VolatileImage createCompatibleImage(int width, int height)
    {
        return Util.getGraphicsConfiguration().createCompatibleVolatileImage(width, height);
    }
    public void rebuildImage()
    {
        buffer = Util.getGraphicsConfiguration().createCompatibleVolatileImage(width, height, Transparency.OPAQUE);
        bufGraphics = buffer.getGraphics();
    }
    public BufferedImage getImage()
    {
        BufferedImage img = new BufferedImage(buffer.getWidth(), buffer.getHeight(), BufferedImage.TYPE_INT_RGB);
        view.draw(img.createGraphics() ,buffer.getWidth(), buffer.getHeight(), null);
        return img;
    }
    @Override
    public void paintComponent(Graphics g)
    {
        Rectangle clip = (Rectangle) ((Graphics2D)g).getClip();
        do
        {
            int code = buffer.validate(Util.getGraphicsConfiguration());
            if(code == VolatileImage.IMAGE_INCOMPATIBLE)
                rebuildImage();
            if(bufGraphics == null)
                bufGraphics = buffer.getGraphics();
            bufGraphics.setClip(clip);
            view.draw((Graphics2D) bufGraphics, getWidth(), getHeight(), clip);

            if(clip.height == height)
                g.drawImage(buffer, 0, 0, null);
            else
                g.drawImage(buffer, 0, 55,  width, height, 0, 55, width, height, null);
        }
        while(buffer.contentsLost());
    }
    @Override
    public void update(Graphics g)
    {
        paintComponent(g);
    }
    @Override
    public void addNotify() {
        super.addNotify();
        UI.viewVisible = true;
    }
    @Override
    public void setCursor(Cursor c)
    {
        cursor = c;
        if(!UI.mouseOverUI)
            super.setCursor(c);
    }
    public void mouseOverUI()
    {
        super.setCursor(Cursor.getDefaultCursor());
    }
    public void mouseOffUI()
    {
        super.setCursor(cursor);
    }
    @Override
    public int getWidth()
    {
        if(super.getWidth() <= 0)
            return width;
        else
            return super.getWidth();
    }
    @Override
    public int getHeight()
    {
        if(super.getHeight() <= 0)
            return height;
        else
            return super.getHeight();
    }
}
