/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package net.buildism.ui.edge;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.io.IOException;
import java.net.URLEncoder;
import net.buildism.AssetManager;
import net.buildism.BuildismException;
import net.buildism.EditorView;
import net.buildism.Main;
import net.buildism.PropertyDef;
import net.buildism.Util;
import net.buildism.component.CommandBar;
import net.buildism.ui.UI;


public final class EditorOverlay extends EMain{
    public PresetsFrame presetsFrame;

    public EButton addRemovePlayerItem;
    public EButton showHideToolsItem;
    public EButton showHideConsoleItem;
    public EButton presetsButton;

    public ESwingFrame tableFrame;
    public ESwingFrame treeFrame;
    public EStickySwingFrame commandBar;
    public EFrame helpFrame;
    public EFrame newFrame;
    public EStickyFrame toolsBackground;
    public EFrame bottomFrame;
    public ETabFrame toolsTabFrame;
    public EScriptingFrame scriptingFrame;
    public EButton wikiButton;
    public EButton deleteButton;

    private CommandBar commandBarComponent = new CommandBar();
    private ELeaderboard leaderboard;

    public EditorOverlay(EditorView view, int width, int height)
    {
        super(width, height);
        final EditorTop top = (EditorTop) view.getTop();
        presetsFrame = new PresetsFrame(this);
        presetsFrame.visible = false;
        try {
            showHideToolsItem = new EButton(AssetManager.loadImage("tools.png", "tools.png"));
            showHideToolsItem.setTooltip("Show/hide advanced tools");
            showHideToolsItem.setUndecorated(true);
            showHideToolsItem.setWidth(32);
            showHideToolsItem.setHeight(32);
            showHideToolsItem.setY(200);
            showHideToolsItem.setActionListener(new EActionListener() {

                @Override
                public void actionPerformed() {
                    if (UI.advancedToolsVisible) {
                        hideAdvancedTools();
                    } else {
                        showAdvancedTools();
                    }
                }
            });
            addChild(1, showHideToolsItem);
            showHideConsoleItem = new EButton(AssetManager.loadImage("lua.png", "lua.png"));
            showHideConsoleItem.setTooltip("Show/hide scripting console");
            showHideConsoleItem.setUndecorated(true);
            showHideConsoleItem.setWidth(32);
            showHideConsoleItem.setHeight(32);
            showHideConsoleItem.setY(250);
            showHideConsoleItem.setActionListener(new EActionListener() {

                @Override
                public void actionPerformed() {
                    if (commandBar.visible) {
                        commandBar.setVisible(false);
                    } else {
                        commandBar.setVisible(true);
                    }
                    Main.repaintAdvancedTools();
                }
            });
            addChild(1, showHideConsoleItem);
            addRemovePlayerItem = new EButton(AssetManager.loadImage("addplayer.png", "addplayer.png"));
            addRemovePlayerItem.setTooltip("Add/remove character");
            addRemovePlayerItem.setUndecorated(true);
            addRemovePlayerItem.setWidth(32);
            addRemovePlayerItem.setHeight(32);
            addRemovePlayerItem.setY(150);
            addRemovePlayerItem.setActionListener(new EActionListener() {

                @Override
                public void actionPerformed() {
                    if(Main.isServer())
                        return;
                    if (Main.getLocalPlayer() == null) {
                        Main.addPlayer(Main.getLoginName());
                    } else {
                        Main.removePlayer();
                    }
                }
            });
            addChild(1, addRemovePlayerItem);
            presetsButton = new EButton(AssetManager.loadImage("addpackage.png", "addpackage.png"));
            presetsButton.setTooltip("Add premade item");
            presetsButton.setUndecorated(true);
            presetsButton.setWidth(32);
            presetsButton.setHeight(32);
            presetsButton.setY(300);
            presetsButton.setActionListener(new EActionListener() {

                @Override
                public void actionPerformed() {
                    if(presetsFrame.visible)
                    {
                        presetsFrame.visible = false;
                    }
                    else
                    {
                        System.out.println("To add basic shapes, click the Advanced Tools button and use the appropriate toolbar buttons.");
                        presetsFrame.visible = true;
                    }
                }
            });

            addChild(1, presetsButton);
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
        
        toolsBackground = new EStickyFrame(this, 220, -1, 0, -1, 55, 0);
        toolsBackground.setUndecorated(true);
        toolsBackground.setBackgroundColor(new Color(192, 192, 192));
        toolsBackground.visible = false;
        
        treeFrame = new ESwingFrame(toolsBackground, 10, 5, 200, 300, view.getPartsTree());
        toolsTabFrame = new ETabFrame(toolsBackground, 10, 385, 200, 20, 280);
        toolsTabFrame.setLayer(1);
        ESwingFrame propsFrame = new ESwingFrame(this, 0, 0, 0, 0, view.getPropsTable());
        toolsTabFrame.addTab("Basic", propsFrame, new EActionListener()
        {

            @Override
            public void actionPerformed() {
                try {
                    UI.propClass = PropertyDef.CLASS_BASIC;
                    UI.updatePropsTable();
                } catch (BuildismException ex) {
                    ex.printStackTrace();
                }
            }            
        });
        toolsTabFrame.addTab("Advanced", propsFrame, new EActionListener()
        {

            @Override
            public void actionPerformed() {
                try {
                    UI.propClass = PropertyDef.CLASS_ADVANCED;
                    UI.updatePropsTable();
                } catch (BuildismException ex) {
                    ex.printStackTrace();
                }
            }            
        });
        scriptingFrame = new EScriptingFrame(this, 0, 0, 0, 0);
        toolsTabFrame.addTab("Scripting", scriptingFrame);
        toolsTabFrame.setTab("Basic");
        toolsTabFrame.setVisible(false);
        
        wikiButton = new EButton("Look Up on Wiki");
        bottomFrame = new EFrame(toolsBackground, 10, 0, 200, 20)
        {
            @Override
            public void render(Graphics2D g)
            {
                renderChildren(g);
            }
        };
        bottomFrame.setVisible(false);
        try {
            deleteButton = new EButton(AssetManager.loadImage("delete.png", "delete.png"));
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        deleteButton.setWidth(20);
        deleteButton.setHeight(20);
        deleteButton.setX(200-20);
        wikiButton.setWidth(200-22);
        wikiButton.setHeight(20);
        deleteButton.setActionListener(new EActionListener()
        {
            @Override
            public void actionPerformed() {
                try {
                    UI.deleteSelection();
                } catch (BuildismException ex) {
                    ex.printStackTrace();
                }
            }
        });
        wikiButton.setActionListener(new EActionListener()
        {

            @Override
            public void actionPerformed() {
                if(UI.getNumberSelected() == 1)
                    Util.wikiLookup(UI.getSelected().get(0).getType());
            }
            
        });
        bottomFrame.addChild(1, wikiButton);
        bottomFrame.addChild(1, deleteButton);
        bottomFrame.setVisible(false);
        
        addChild(0, toolsBackground);
        commandBar = new EStickySwingFrame(this, 700, 40, -1, 0, 0, -1, commandBarComponent);
        commandBar.setUndecorated(true);
        commandBar.setBackgroundColor(new Color(192, 192, 192));
        addChild(1, commandBar);
        setSizes();
        
        helpFrame = new EFrame(this, 200, 125, 400, 250);
        helpFrame.addTitle("Buildism Help");
        ELabel controlsLabel = new ELabel("Editor Controls");
        controlsLabel.setColor(new Color(197, 203, 217));
        controlsLabel.setFont(new Font("Arial", Font.BOLD, 16));
        helpFrame.addChild(1, controlsLabel);
        controlsLabel.setX(6);
        controlsLabel.setY(28);
        /*String[] lines = {"* a, left arrow - move left",
        "* d, right arrow - move right",
        "* shift - run",
        "* w, up arrow - jump",
        "* 1-9 - select tool",
        "* Q - drop tool (if applicable)"};*/
        String[] lines = {"* left mouse button- select",
        "* right mouse button - move camera",
        "* mouse wheel or (+), (-) - zoom",
        "* shift - multi select",
        "* delete - delete selection"};
        int y = 47;
        Font lineFont = new Font("Arial", -1, 13);
        for(String l : lines)
        {
            ELabel lineLabel = new ELabel(l);
            lineLabel.setFont(lineFont);
            lineLabel.setX(6);
            lineLabel.setY(y);
            helpFrame.addChild(1, lineLabel);
            y += 17;
        }
        helpFrame.visible = false;
        
        addChild(7, helpFrame);
        leaderboard = new ELeaderboard(this, 230, 60);
    }
    public void showNewFrame()
    {
        if(newFrame != null && newFrame.visible)
            return;
        if(hasChild(1, newFrame))
            this.removeChild(newFrame);
        newFrame = new EStickyFrame(this, -1, -1, 40, 10, 60, 10);
        newFrame.visible = true;
        newFrame.addTitle("New Game");
        
        ELabel sideLabel = new ELabel("Side-scrolling");
        sideLabel.setColor(new Color(197, 203, 217));
        sideLabel.setFont(new Font("Arial", Font.BOLD, 16));
        sideLabel.setX(6);
        sideLabel.setY(30);
        newFrame.addChild(1, sideLabel);
        
        addTemplate(10, 50, "Classic");
        addTemplate(240, 50, "Big");
        addTemplate(470, 50, "Tall");
        addTemplate(10, 190, "Islands");
        addTemplate(240, 190, "Moon");
        addTemplate(470, 190, "Ocean");
        
        ELabel topLabel = new ELabel("Top down");
        topLabel.setColor(new Color(197, 203, 217));
        topLabel.setFont(new Font("Arial", Font.BOLD, 16));
        topLabel.setX(6);
        topLabel.setY(330);
        newFrame.addChild(1, topLabel);
        addTemplate(10, 350, "Driving");
        addTemplate(240, 350, "Empty");
    }
    public void hideAdvancedTools()
    {
        toolsBackground.setVisible(false);
        UI.advancedToolsVisible = false;
    }
    public void showAdvancedTools()
    {
        toolsBackground.setVisible(true);
        UI.advancedToolsVisible = true;
    }
    private void addTemplate(int x, int y, final String name)
    {
        try {
            EButton b = new EButton(AssetManager.loadImage("Template_"+name, "resource://templates/"+name+".jpg"), 210, 120);
            b.setTooltip(name.replace("_", " "));
            b.setX(x);
            b.setY(y);
            newFrame.addChild(1, b);
            b.setActionListener(new EActionListener()
            {
                @Override
                public void actionPerformed() {
                    if(Main.loading)
                        return;
                    try {
                        Main.newGame();
                        Main.getGame().setMainWorld();
                        Main.getGame().stop(true);
                        Main.loadFromResource("/net/resources/templates/"+name, true);
                    } catch (IOException ex) {
                        Main.newGame();
                        Main.getGame().setMainWorld();
                    }
                    System.gc();
                    newFrame.visible = false;
                }
            });
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
    }
    public void setSizes()
    {
        int height = toolsBackground.getHeight();
        int padding = 50;
        int boxHeight = (height - padding)/2;
        treeFrame.setHeight(boxHeight);
        toolsTabFrame.setY(boxHeight+padding/2-10);
        toolsTabFrame.setHeight(boxHeight);
        bottomFrame.setY(boxHeight*2+padding/2);
    }
    @Override
    public boolean eMouseDown(int b, int x, int y)
    {
        boolean r = super.eMouseDown(b, x, y);
        if(x > getWidth() - 220)
            Main.repaintAdvancedTools();
        return r;
    }
    @Override
    public boolean eMouseUp(int b, int x, int y)
    {
        boolean r = super.eMouseUp(b, x, y);
        if(x > getWidth() - 220)
            Main.repaintAdvancedTools();
        return r;
    }
}
