/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * LauncherFrame.java
 *
 * Created on May 31, 2011, 3:51:20 PM
 */

package net.buildism.launcher;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;

/**
 *
 * @author Jacob Morgan
 */
public class LauncherFrame extends javax.swing.JFrame {
    private BufferedImage bg;
    private Color bgColor = new Color(235, 235, 235);
    private BufferedImage logo;
    public TexturePaint paint;
    private JPanel cornerPanel;
    private String gameArguments;
    private Properties properties = new Properties();
    public static String gamePath;
    private LauncherFrame instance;
    /** Creates new form LauncherFrame */
    public LauncherFrame() {
        super("Buildism");
        try
        {
            setIconImage(new ImageIcon(getClass().getResource("/buildismicon.png")).getImage());
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
        String cacheDir = System.getProperty("deployment.user.cachedir");
        if (cacheDir == null || System.getProperty("os.name").startsWith("Win"))
                cacheDir = System.getProperty("java.io.tmpdir");
        gamePath = cacheDir+"/.buildism/";
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
        this.setContentPane(new JPanel()
        {
            @Override
            public void paintComponent(Graphics g)
            {
                Graphics2D g2 = (Graphics2D) g;
                Paint old = g2.getPaint();
                g2.setPaint(paint);
                g2.fillRect(0, 0, getWidth(), getHeight());
                g2.setPaint(old);
                g2.setColor(bgColor);
                g2.fillRect(0, 0, 266, 88);
                g2.drawImage(logo, 20, 20, this);
                paintChildren(g);
            }
        });
        File gamePathFile = new File(gamePath);
        boolean exists = gamePathFile.exists();
        if(!exists)
        {
            gamePathFile.mkdir();
            File libFile = new File(gamePath+"/lib");
            libFile.mkdir();
        }
        try {
            properties.load(new FileInputStream(gamePath+"/settings.properties"));
        }
        catch(IOException ex)
        {
            properties.setProperty("username", "Username");
            properties.setProperty("password", "Password");
            properties.setProperty("remember", "false");
        }
        initComponents();
        usernameField.setText(properties.getProperty("username"));
        passwordField.setText(properties.getProperty("password"));
        rememberCheckbox.setSelected(Boolean.parseBoolean(properties.getProperty("remember")));
        cornerPanel = loginPanel;
        try {
            bg = ImageIO.read(LauncherFrame.class.getResource("/circlebg.jpg"));
            paint = new TexturePaint(bg, new Rectangle(0, 0, 1000, 600));
            logo = ImageIO.read(LauncherFrame.class.getResource("/logo.png"));
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        instance = this;
    }

    private void run(Runnable r)
    {
        new Thread(r).start();
    }

    private void login()
    {
        setCornerFrame(new LoggingInPanel());
        run(new Runnable()
        {

            public void run() {
                try {
                    String username = usernameField.getText();
                    String password = String.copyValueOf(passwordField.getPassword());
                    String result = LauncherUtil.httpGet("http://buildism.net/game/login.php?user="+username+"&pass="+password).trim();
                    String[] parts = result.split(";");
                    String[] login = parts[0].split("\\|");
                    int uid = Integer.parseInt(login[0]);
                    String key = login[1];
                    if(uid == 0)
                        loginFailed();
                    else
                    {
                        boolean checked = rememberCheckbox.getSelectedObjects() != null;
                        if(checked)
                        {
                            properties.setProperty("username", username);
                            properties.setProperty("password", password);
                        }
                        else
                        {
                            properties.setProperty("username", "Username");
                            properties.setProperty("password", "Password");
                        }
                        properties.setProperty("remember", Boolean.toString(checked));
                        FileOutputStream out = new FileOutputStream(gamePath+"/settings.properties");
                        properties.store(out, "Buildism launcher properties");
                        out.close();

                        gameArguments = uid+" "+username+" "+key;
                        UpdaterApplet updater = new UpdaterApplet(getWidth(), getHeight(), gameArguments, true);
                        updater.setParameter("urls", parts[1]);
                        updater.init();
                        setContentPane(updater);
                        validate();
                        updater.start();
                    }
                } catch (IOException ex) {
                    setCornerFrame(new ConnectionErrorPanel(instance));
                }
            }

        });
    }

    public void tryAgain()
    {
        setCornerFrame(loginPanel);
    }

    public boolean isGameDownloaded()
    {
        return new File(gamePath+"/urls").exists();
    }

    public void playOffline()
    {
        gameArguments = "0 Player X";
        try {
            FileInputStream in = new FileInputStream(gamePath + "/urls");
            BufferedReader r = new BufferedReader(new InputStreamReader(in));
            String urls = r.readLine();
            UpdaterApplet updater = new UpdaterApplet(getWidth(), getHeight(), gameArguments, false);
            updater.setParameter("urls", urls);
            updater.init();
            setContentPane(updater);
            validate();
            updater.start();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    private void loginFailed()
    {
        setCornerFrame(new LoginFailedPanel());
        try {
            Thread.sleep(2000);
        } catch (InterruptedException ex) {
        }
        setCornerFrame(loginPanel);
    }

    private void setCornerFrame(JPanel panel)
    {
        cornerContainer.remove(cornerPanel);
        cornerPanel = panel;
        cornerContainer.add(panel);
        panel.setSize(203, 117);
        cornerContainer.setPreferredSize(new Dimension(203, 117));
        validate();
        repaint();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        cornerContainer = new javax.swing.JPanel();
        loginPanel = new javax.swing.JPanel();
        usernameField = new javax.swing.JTextField();
        loginButton = new javax.swing.JButton();
        rememberCheckbox = new javax.swing.JCheckBox();
        passwordField = new javax.swing.JPasswordField();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        loginPanel.setBackground(new java.awt.Color(235, 235, 235));

        usernameField.setText("Username");

        loginButton.setText("Log in");
        loginButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loginButtonActionPerformed(evt);
            }
        });

        rememberCheckbox.setText("Remember");

        passwordField.setText("Password");

        javax.swing.GroupLayout loginPanelLayout = new javax.swing.GroupLayout(loginPanel);
        loginPanel.setLayout(loginPanelLayout);
        loginPanelLayout.setHorizontalGroup(
            loginPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(loginPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(loginPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(passwordField, javax.swing.GroupLayout.DEFAULT_SIZE, 183, Short.MAX_VALUE)
                    .addComponent(usernameField, javax.swing.GroupLayout.DEFAULT_SIZE, 183, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, loginPanelLayout.createSequentialGroup()
                        .addComponent(rememberCheckbox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(loginButton)))
                .addContainerGap())
        );
        loginPanelLayout.setVerticalGroup(
            loginPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(loginPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(usernameField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(passwordField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(loginPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(loginButton)
                    .addComponent(rememberCheckbox))
                .addContainerGap(15, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout cornerContainerLayout = new javax.swing.GroupLayout(cornerContainer);
        cornerContainer.setLayout(cornerContainerLayout);
        cornerContainerLayout.setHorizontalGroup(
            cornerContainerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(loginPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        cornerContainerLayout.setVerticalGroup(
            cornerContainerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, cornerContainerLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(loginPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(466, Short.MAX_VALUE)
                .addComponent(cornerContainer, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(347, Short.MAX_VALUE)
                .addComponent(cornerContainer, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void loginButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loginButtonActionPerformed
        login();
    }//GEN-LAST:event_loginButtonActionPerformed

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                LauncherFrame frame = new LauncherFrame();
                frame.setSize(1024, 768);
                frame.setLocationRelativeTo(null);
                frame.setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel cornerContainer;
    private javax.swing.JButton loginButton;
    private javax.swing.JPanel loginPanel;
    private javax.swing.JPasswordField passwordField;
    private javax.swing.JCheckBox rememberCheckbox;
    private javax.swing.JTextField usernameField;
    // End of variables declaration//GEN-END:variables

}
