/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package net.buildism.launcher;

import java.applet.Applet;
import java.applet.AppletStub;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Method;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.SocketPermission;
import java.net.URL;
import java.net.URLClassLoader;
import java.security.AccessController;
import java.security.CodeSource;
import java.security.PermissionCollection;
import java.security.PrivilegedAction;
import java.security.SecureClassLoader;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.UIManager;



public final class UpdaterApplet extends JApplet implements AppletStub{
    private int w;
    private int h;
    private String options;
    private boolean launcher = false;
    private boolean update;
    private boolean started = false;
    private boolean loading = false;
    private ProgressBarInputStream downloadStream;
    private UpdaterPanel updatePanel;
    private BufferedImage bg;
    private BufferedImage loadingImage;
    public TexturePaint paint;
    private HashMap<String, String> parameters = new HashMap<String, String>(1);
    private String[] urls;
    private String error;
    private UpdaterApplet instance;
    private String gamePath;
    private boolean force = false;
    public UpdaterApplet()
    {
        w = getWidth();
        h = getHeight();
        update = true;
        instance = this;
        String cacheDir = System.getProperty("deployment.user.cachedir");
        if (cacheDir == null || System.getProperty("os.name").startsWith("Win"))
                cacheDir = System.getProperty("java.io.tmpdir");
        gamePath = cacheDir+"/.buildism/";
        File gamePathFile = new File(gamePath);
        boolean exists = gamePathFile.exists();
        if(!exists)
        {
            gamePathFile.mkdir();
            File libFile = new File(gamePath+"/lib");
            libFile.mkdir();
        }
    }
    public UpdaterApplet(int w, int h, String options, boolean doUpdate)
    {
        setSize(w, h);
        launcher = true;
        this.options = options;
        update = doUpdate;
        setParameter("options", options);
        setParameter("urls", "http://buildism.net/game/Buildism.jar|http://buildism.net/game/lib/resources.jar|http://buildism.net/game/lib/jsyntaxpane-0.9.5-b29.jar");
        instance = this;
        String cacheDir = System.getProperty("deployment.user.cachedir");
        if (cacheDir == null || System.getProperty("os.name").startsWith("Win"))
                cacheDir = System.getProperty("java.io.tmpdir");
        gamePath = cacheDir+"/.buildism/";
    }

    public void setParameter(String key, String value)
    {
        parameters.put(key, value);
    }
    @Override
    public String getParameter(String k)
    {
        if(parameters.containsKey(k))
            return parameters.get(k);
        else
            try {
            return super.getParameter(k);
            }
            catch(Exception ex)
            {
                return null;
            }
    }

    @Override
    public void init()
    {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

        try {
            bg = ImageIO.read(UpdaterApplet.class.getResource("/circlebg.jpg"));
            paint = new TexturePaint(bg, new Rectangle(0, 0, 1000, 600));
            loadingImage = ImageIO.read(UpdaterApplet.class.getResource("/loading.png"));
        } catch (IOException ex) {
            ex.printStackTrace();
        }

        this.setContentPane(new JPanel()
        {
            @Override
            public void paintComponent(Graphics g)
            {
                if(!started)
                {
                    Graphics2D g2 = (Graphics2D) g;
                    Paint old = g2.getPaint();
                    g2.setPaint(paint);
                    g2.fillRect(0, 0, getWidth(), getHeight());
                    g2.setPaint(old);
                    paintChildren(g);
                    if(error != null)
                    {
                        g2.setColor(Color.white);
                        g2.drawString(error, 100, 100);
                    }
                    if(loading)
                    {
                        g2.drawImage(loadingImage, getWidth()/2 - loadingImage.getWidth()/2, getHeight()/2 - loadingImage.getHeight()/2, null);
                    }
                }
                else
                    super.paintComponent(g);
            }
        });

        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        updatePanel = new UpdaterPanel();
        add(updatePanel, c);
        
        invalidate();
        repaint();

        urls = this.getParameter("urls").split("\\|");
    }
    private void error(Exception ex)
    {
        error = "Unexpected error: "+ex.toString()+" Redownloading the game may fix the problem.";
        loading = false;
        started = false;
        JButton b = new JButton("Redownload");
        b.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e) {
                if(Desktop.isDesktopSupported())
                {
                    try {
                        Desktop.getDesktop().browse(new URL("http://buildism.net/editor.php?update").toURI());
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
            }
        });
        add(b);
        validate();
        setIgnoreRepaint(false);
        repaint();
        System.err.println("Error starting Buildism");
        ex.printStackTrace();
    }
    private void startGame(final URL[] urls)
    {
        remove(updatePanel);
        invalidate();
        loading = true;
        repaint();
        try {
            EventQueue.invokeAndWait(new Runnable() {

                public void run() {
                    URLClassLoader loader = new URLClassLoader(urls) {

                        @Override
                        protected PermissionCollection getPermissions(CodeSource codeSource) {
                            PermissionCollection perms = null;
                            try {
                                Method method = SecureClassLoader.class.getDeclaredMethod("getPermissions", new Class[]{CodeSource.class});
                                method.setAccessible(true);
                                perms = (PermissionCollection) method.invoke(getClass().getClassLoader(), new Object[] {codeSource});
                                if(!launcher)
                                {
                                    String host = getCodeBase().getHost();
                                    perms.add(new SocketPermission(host, "connect,accept"));
                                }
                            } catch (Exception ex) {
                                ex.printStackTrace();
                            }
                            return perms;
                        }
                    };
                    Thread.currentThread().setContextClassLoader(loader);
                    try {
                        Class mainClass = loader.loadClass("net.buildism.MainApplet");
                        Applet applet = (Applet) mainClass.newInstance();
                        applet.setStub(instance);
                        applet.setSize(getWidth(), getHeight());
                        applet.init();
                        started = true;
                        setIgnoreRepaint(true);
                        setContentPane(applet);
                        applet.start();
                    } catch (Exception ex) {
                        error(ex);
                    }
                }
            });
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    @Override
    public void start()
    {
        AccessController.doPrivileged(new PrivilegedAction()
        {
            public Object run() {
                update();
                return null;
            }
        });
    }

    private void update()
    {
        int version = 0;
        try {
            FileInputStream file = new FileInputStream(gamePath+"/version ");
            version = Integer.parseInt(new BufferedReader(new InputStreamReader(file)).readLine().trim());
            file.close();
        } catch (IOException ex) {

        } catch(Exception ex) {
            error(ex);
        }
        int currentVersion = 0;
        try {
            currentVersion = Integer.parseInt(LauncherUtil.httpGet("http://buildism.net/game/version.txt").trim());
            if(update && (version != currentVersion || urls[0].contains("test") || getParameter("force") != null))
            {
                DataOutputStream versionOut = new DataOutputStream(new FileOutputStream(gamePath+"/version"));
                versionOut.write(Integer.toString(currentVersion).getBytes());
                versionOut.close();
                new Thread(new Runnable()
                {
                    public void run() {
                        int i = 1;
                        URL[] fileURLs = new URL[urls.length];
                        try {
                            DataOutputStream dout = new DataOutputStream(new FileOutputStream(gamePath + "/urls"));
                            for(String urlString : urls)
                            {
                                dout.write((urlString+"|").getBytes());
                                updatePanel.fileLabel.setText("File "+i+"/"+urls.length);
                                try {
                                    URL url = new URL(urlString);
                                    HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                                    conn.setUseCaches(false);
                                    int len = conn.getContentLength();
                                    downloadStream = new ProgressBarInputStream(conn.getInputStream(), updatePanel.progressBar, len);
                                    String filePath = gamePath + "/" + urlString.replace("http://buildism.net/game/", "");
                                    File file = new File(filePath);
                                    file.delete();
                                    FileOutputStream out = new FileOutputStream(filePath);
                                    byte[] buffer = new byte[8192];
                                    int read = 0;
                                    while ((read = downloadStream.read(buffer)) > 0) {
                                        out.write(buffer, 0, read);
                                    }
                                    out.close();
                                    downloadStream.close();
                                    fileURLs[i - 1] = new File(filePath).toURI().toURL();
                                } catch (Exception ex) {
                                    updatePanel.updateLabel.setText("Unexpected error: "+ex.toString());
                                }
                                i++;
                            }
                            dout.close();
                        } catch (Exception ex) {
                            error(ex);
                        }
                        startGame(fileURLs);
                    }
                }).start();
            }
            else
            {
                new Thread(new Runnable()
                {
                    public void run() {
                        URL[] fileURLs = new URL[urls.length];
                        int i = 0;
                        for(String urlString : urls)
                        {
                            String filePath = gamePath + "/" + urlString.replace("http://buildism.net/game/", "");
                            try {
                                fileURLs[i] = new File(filePath).toURI().toURL();
                            } catch (MalformedURLException ex) {
                                ex.printStackTrace();
                            }
                            i++;
                        }
                        repaint();
                        try {
                            Thread.sleep(200);
                        } catch (InterruptedException ex) {
                            ex.printStackTrace();
                        }
                        startGame(fileURLs);
                    }
                }).start();
            }
        } catch (IOException ex) {
            error(ex);
        }
    }

    public void appletResize(int width, int height) {
        resize(width, height);
    }
}
